# -*- coding:utf-8 -*-
#
# author: philip1134
# date: 2021-07-15
#


from pyzentao import Zentao


if "__main__" == __name__:
    zentao = Zentao({
        # zentao root url
        "url": "http://0.0.0.0:21761/zentao",
        "version": "17.6",

        # authentication
        "username": "admin",
        "password": "Lton2008@",
    })

    response = zentao.user_task(
        userID=1,
        type="assignedTo",
        # raw=True
    )

    if response.status == "success":
        tasks = sorted(response.data["tasks"].values(),
                       key=lambda x: int(x.get("id")))

        print(">" * 30)
        print(tasks)
        print("<" * 30)

# end



# -*- coding:utf-8 -*-
#
# author: philip1134
# date: 2021-07-15
#


import json
from pyzentao.attribute_dict import AttributeDict


class Response:
    """zentao api response"""

    def __init__(self, raw):
        super(Response, self).__init__()

        self.raw = raw
        self.status = None
        self.data = {}

        self.parse()

    def __str__(self):
        """printer"""

        text = ""
        for attr in ("status", "data",):
            text += "%s: %s\n" % (
                attr, str(self.__dict__[attr]))

        return text

# public
    def parse(self):
        """parse raw data, just keep 'status' and 'data'"""

        if isinstance(self.raw, dict):
            self.status = self.raw.get("status", None)
            self.data = AttributeDict(
                json.loads(self.raw.get("data", {}))
            )


# end



# -*- coding:utf-8 -*-
#
# author: philip1134
# date: 2021-07-15
#


import json
import requests
from pyzentao.common import get_json
from pyzentao.exceptions import AuthenticationError


class Session:
    """zentao connection session"""

    def __init__(
        self,
        username,
        password,
        session_api,
        login_api,
    ):
        super(Session, self).__init__()

        self.username = username
        self.password = password
        self.session_api = session_api
        self.login_api = login_api

        self.connected = False
        self.name = None
        self.id = None

# public
    def connect(self):
        """setup connection"""

        if not self.connected:
            self.connected = self._get_session() and self._login()

        return self.connected

# protected
    def _get_session(self):
        """get zentao session name and session id"""

        response = get_json(requests.get(
            self.session_api.get("url")
        ))

        if "success" == response.get("status"):
            # get data list and return to caller
            session = json.loads(response["data"])
            self.name = session["sessionName"]
            self.id = session["sessionID"]
            return True
        else:
            # fail to get session
            raise AuthenticationError("Fail to get session")

    def _login(self):
        """login zentao with username and password"""

        response = get_json(requests.post(
            self.login_api.get("url"),
            params={
                "account": self.username,
                "password": self.password,
                self.name: self.id
            }
        ))

        if "success" == response.get("status"):
            return True
        else:
            # fail to sign in
            raise AuthenticationError(
                "Fail to sign in Zentao with account '%s', password '%s'" % (
                    self.username, self.password))


# end



# -*- coding:utf-8 -*-
#
# author: philip1134
# date: 2021-08-01
#


_splitter = "-" * 20


class PyZentaoException(Exception):
    """pyzentao base exception class"""

    pass


class AuthenticationError(PyZentaoException):
    """exception for authentication error"""

    pass


class APINameError(PyZentaoException):
    """exception for unknown api name error"""

    def __init__(self, name):
        super(APINameError, self).__init__()

        self.name = name

    def __str__(self):
        """printer"""

        return "Unknown API name '%s'" % self.name


class InvalidJSONResponseError(PyZentaoException):
    """exception for invalid json in response error"""

    def __init__(self, response):
        super(InvalidJSONResponseError, self).__init__()

        self.response = response

    def __str__(self):
        """printer"""

        return """
Invalid object type or invalid json in api response, that may raise
json.decoder.JSONDecodeError, the original response is:
%s\n%s\n%s\n""" % (_splitter, self.response, _splitter)


# end



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import re
from setuptools import setup

with io.open('README.rst', 'rt', encoding='utf8') as f:
    readme = f.read()

with io.open('pyzentao/__init__.py', 'rt', encoding='utf8') as f:
    version = re.search(r'__version__ = \"(.*?)\"', f.read()).group(1)

setup(
    name='pyzentao',
    version=version,
    url='https://github.com/philip1134/pyzentao',
    license='MIT',
    author='Philip CHAN',
    author_email='philip1134@imior.com',
    description='Python SDK for Zentao.',
    long_description=readme,
    packages=['pyzentao'],
    package_data={
        '': ['*.md', '*.rst'],
    },
    platforms='any',
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "requests",
        "PyYAML",
    ],
    extras_require={
        'dev': [
            'tox',
            'check-manifest',
            'flake8'
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    keywords='Python SDK',
)



# -*- coding:utf-8 -*-
#
# author: philip1134
# date: 2021-11-07
#


import json
import requests
from pyzentao.exceptions import InvalidJSONResponseError


def get_json(response):
    """get json from response"""

    try:
        if isinstance(response, requests.Response):
            return response.json()
        else:
            # invalid object type
            raise InvalidJSONResponseError(response)
    except json.decoder.JSONDecodeError:
        # invalid json in api response
        raise InvalidJSONResponseError(response) from None


# end
